/***************************************************************************
 *   Copyright (C) 2007-2011 by Vegeta                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef Preferences_H
#define Preferences_H

#include "ui_Preferences.h"
#include "ui_QtGain.h"

using namespace Ui;

class Preferences : public QDialog, public PreferencesClass
{
    Q_OBJECT

public:
    Preferences(QWidget *parent = 0);
    ~Preferences();
    void setIcons();

private slots:
    void on_SelectMP3Binary_clicked();
    void on_SelectMP4Binary_clicked();
    void on_SelectVorbisBinary_clicked();
    void on_SelectFlacBinary_clicked();
    void on_pushButton_Reset_clicked();
    void on_Selectid3v2Binary_clicked();
};
#endif // Preferences_H
